/***************************************************************
 * Name:      idwMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Wolfgang Gossel (wolfgang.gossel@geo.uni-halle.de)
 * Created:   2013-09-27
 * Copyright: Wolfgang Gossel (umweltgeologie.geologie.uni-halle.de)
 * License:   GPL
 **************************************************************/
#include "calc_idw.h"

calculate_IDW::calculate_IDW()
{
    //ctor
}

calculate_IDW::~calculate_IDW()
{
    //dtor
}

void calculate_IDW::setRaster(double xRasMin, double yRasMin, double xRasMax, double yRasMax, double rasWidth)
{
    long counterRows, counterCols;
    numberRasterCols=((xRasMax-xRasMin)/rasWidth)+0.5;
    numberRasterRows=((yRasMax-yRasMin)/rasWidth)+0.5;

    for(counterRows=0; counterRows<numberRasterRows; counterRows++)
    {
        for(counterCols=0; counterCols<numberRasterCols;counterCols++)
        {
            xRaster.push_back(xRasMin+counterCols*rasWidth);
            yRaster.push_back(yRasMin+counterRows*rasWidth);
            zRaster.push_back(0);
        }

    }
}

void calculate_IDW::setPointData(vector<double> &xData, vector<double> &yData, vector<double> &zData)
{
    long counter;
    for(counter=0;counter<xData.size(); counter++)
    {
        xMeasure.push_back(xData[counter]);
        yMeasure.push_back(yData[counter]);
        zMeasure.push_back(zData[counter]);
    }

}

void calculate_IDW::calculateAllMeasPoints(long powIDW)
{
    long counterRasPoints=0, counterMeasPoints=0, numRasPoints, numMeasurePoints;
    double sumDist=0, sumDistValue=0, distX=0, distY, distTot;

    powerIDW=powIDW;


    for(counterRasPoints=0; counterRasPoints<xRaster.size(); counterRasPoints++)
    {
        for (counterMeasPoints=0; counterMeasPoints<xMeasure.size(); counterMeasPoints++)
        {
            distX=xRaster[counterRasPoints]-xMeasure[counterMeasPoints];
            distX=distX*distX;
            distY=yRaster[counterRasPoints]-yMeasure[counterMeasPoints];
            distY=distY*distY;
            distTot=distX+distY;
            distTot=sqrt(distTot);
            switch(powerIDW)
            {
                case 1:
                    distTot=distTot;
                    break;
                case 2:
                    distTot=distTot*distTot;
                    break;
                case 3:
                    distTot=distTot*distTot*distTot;
                    break;
                case 4:
                    distTot=distTot*distTot*distTot*distTot;
                    break;
            }
            sumDist=sumDist+1/distTot;
            sumDistValue=sumDistValue+(zMeasure[counterMeasPoints]/distTot);
            distTot=0;
        }

        zRaster[counterRasPoints]=sumDistValue/sumDist;

        sumDistValue=0;
        sumDist=0;
    }
}

void calculate_IDW::calculateRadiusMeasPoints(long powIDW, double searchRadius)
{
    long counterRasPoints=0, counterMeasPoints=0, numRasPoints, numMeasurePoints;
    double sumDist=0, sumDistValue=0, distX=0, distY, distTot;

    powerIDW=powIDW;


    for(counterRasPoints=0; counterRasPoints<xRaster.size(); counterRasPoints++)
    {
        for (counterMeasPoints=0; counterMeasPoints<xMeasure.size(); counterMeasPoints++)
        {
            distX=xRaster[counterRasPoints]-xMeasure[counterMeasPoints];
            distX=distX*distX;
            distY=yRaster[counterRasPoints]-yMeasure[counterMeasPoints];
            distY=distY*distY;
            distTot=distX+distY;
            distTot=sqrt(distTot);
            if (distTot<=searchRadius)
            {
                switch(powerIDW)
                {
                    case 1:
                        distTot=distTot;
                        break;
                    case 2:
                        distTot=distTot*distTot;
                        break;
                    case 3:
                        distTot=distTot*distTot*distTot;
                        break;
                    case 4:
                        distTot=distTot*distTot*distTot*distTot;
                        break;
                }
                sumDist=sumDist+1/distTot;
                sumDistValue=sumDistValue+(zMeasure[counterMeasPoints]/distTot);
            }
            distTot=0;
        }

        zRaster[counterRasPoints]=sumDistValue/sumDist;

        sumDistValue=0;
        sumDist=0;
    }
}

void calculate_IDW::calculateNumberMeasPoints(long powIDW, long numNeighbors, double searchRadius)
{
    vector<sortForIDW> tempVector;
    long counterRasPoints=0, counterMeasPoints=0, numRasPoints, numMeasurePoints, tmpNumPnt, hsj, hsi, hsir, counterIndexedPoints, index;
    double sumDist=0, sumDistValue=0, distX=0, distY, distTot, a, zSelTmp;

    powerIDW=powIDW;
    numberNeighborPoints=numNeighbors;

    wxString tmpString, tmpString1, tmpString2, tmpString5, tmpString6, tmpString3, tmpString4, tmpString7, tmpString8, tmpString9, tmpString10;

        tmpNumPnt=numberNeighborPoints*xMeasure.size();

        indexMeasureRas = new long[xMeasure.size()];
        distRasterPntMeasure=new double [xMeasure.size()];

        //calculate distances
        for(counterRasPoints=0; counterRasPoints<xRaster.size(); counterRasPoints++)
        {
            for (counterMeasPoints=0; counterMeasPoints<xMeasure.size(); counterMeasPoints++)
            {
                distX=xRaster[counterRasPoints]-xMeasure[counterMeasPoints];
                distX=distX*distX;
                distY=yRaster[counterRasPoints]-yMeasure[counterMeasPoints];
                distY=distY*distY;
                distTot=distX+distY;
                distRasterPntMeasure[counterMeasPoints] =sqrt(distTot);
                indexMeasureRas[counterMeasPoints] =counterMeasPoints;

                if (distRasterPntMeasure[counterMeasPoints]<searchRadius)
                {
                    sortForIDW sortIt;
                    sortIt.distRM=sqrt(distTot);
                    sortIt.idM=counterMeasPoints;
                    sortIt.valueM=zMeasure[counterMeasPoints];
                    tempVector.push_back(sortIt);
                }
            }
//            tmpString1=wxString::Format(wxT("%i"), indexMeasureRas[1]);
//            tmpString2=wxString::Format(wxT("%f"), distRasterPntMeasure[1]);
//            tmpString5=wxString::Format(wxT("%i"), indexMeasureRas[70]);
//            tmpString6=wxString::Format(wxT("%f"), distRasterPntMeasure[70]);
//            tmpString<<wxT("index1; ")<<tmpString1<<wxT("; ")<<tmpString2<<wxT("; index70; ")<<tmpString5<<wxT("; dist70: ")<<tmpString6<<wxT("; ")<<wxT("\n");

 //           wxMessageBox(tmpString);


            //select the nearest measurement points


 //         use sorting algorithm for vectors


//            tmpString=wxT("");
//            for (counterMeasPoints=0; counterMeasPoints<xMeasure.size(); counterMeasPoints++)
//            {
//                if (distRasterPntMeasure[counterMeasPoints]<searchRadius)
//                {
//                    sortForIDW sortIt;
//                    sortIt.distRM=sqrt(distTot);
//                    sortIt.idM=counterMeasPoints;
//                    sortIt.valueM=zMeasure[counterMeasPoints];
//                    tempVector.push_back(sortIt);
//                }
//
//                else {;}
//            }

            sort(tempVector.begin(), tempVector.end(), sortForIDW::sortAscending);

//            for (counterMeasPoints=0; counterMeasPoints<numberNeighborPoints; counterMeasPoints++)
//            {
//                tmpString1=wxString::Format(wxT("%i"), indexMeasureRas[1]);
//                tmpString<<wxT("; "<<tmpString1);
//            }
//
//            wxMessageBox(tmpString);


            for (counterMeasPoints=0; counterMeasPoints<numberNeighborPoints; counterMeasPoints++)
            {
//                index=indexMeasureRas[counterMeasPoints];

                distTot=tempVector[counterMeasPoints].distRM;
                zSelTmp=tempVector[counterMeasPoints].valueM;


                switch(powerIDW)
                {
                    case 1:
                        distTot=distTot;
                        break;
                    case 2:
                        distTot=distTot*distTot;
                        break;
                    case 3:
                        distTot=distTot*distTot*distTot;
                        break;
                    case 4:
                        distTot=distTot*distTot*distTot*distTot;
                        break;
                    case 5:
                        distTot=distTot*distTot*distTot*distTot*distTot;
                        break;
                    case 6:
                        distTot=distTot*distTot*distTot*distTot*distTot*distTot;
                        break;
                    case 7:
                        distTot=distTot*distTot*distTot*distTot*distTot*distTot*distTot;
                        break;
                    case 8:
                        distTot=distTot*distTot*distTot*distTot*distTot*distTot*distTot*distTot;
                        break;
                }
            sumDist=sumDist+(1.0/distTot);
            sumDistValue=sumDistValue+(zSelTmp/distTot);
            distTot=0;
            }

          zRaster[counterRasPoints]=sumDistValue/sumDist;

          sumDistValue=0;
          sumDist=0;
          tempVector.clear();
        }

}

void calculate_IDW::getRasterData(vector<double> &xRasData, vector<double> &yRasData, vector<double> &zRasData)
{
    xRasData=xRaster;
    yRasData=yRaster;
    zRasData=zRaster;
}

long calculate_IDW::getCols()
{

    return numberRasterCols;

}
long calculate_IDW::getRows()
{
    return numberRasterRows;
}

bool sortForIDW::sortAscending(const sortForIDW &a, const sortForIDW &b)
{
    return a.distRM < b.distRM;
}
