/***************************************************************
 * Name:      idwMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Wolfgang Gossel (wolfgang.gossel@geo.uni-halle.de)
 * Created:   2013-09-27
 * Copyright: Wolfgang Gossel (umweltgeologie.geologie.uni-halle.de)
 * License:   GPL
 **************************************************************/

#include "idwMain.h"
#include "getInput.h"
#include "calc_idw.h"
#include "export.h"
#include <wx/msgdlg.h>

//(*InternalHeaders(idwDialog)
#include <wx/artprov.h>
#include <wx/bitmap.h>
#include <wx/intl.h>
#include <wx/image.h>
#include <wx/string.h>
//*)

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

//(*IdInit(idwDialog)
const long idwDialog::ID_STATICTEXT1 = wxNewId();
const long idwDialog::ID_TEXTCTRL1 = wxNewId();
const long idwDialog::ID_BITMAPBUTTON1 = wxNewId();
const long idwDialog::ID_STATICTEXT3 = wxNewId();
const long idwDialog::ID_STATICTEXT2 = wxNewId();
const long idwDialog::ID_STATICTEXT4 = wxNewId();
const long idwDialog::ID_TEXTCTRL8 = wxNewId();
const long idwDialog::ID_TEXTCTRL2 = wxNewId();
const long idwDialog::ID_STATICTEXT6 = wxNewId();
const long idwDialog::ID_STATICTEXT7 = wxNewId();
const long idwDialog::ID_STATICTEXT8 = wxNewId();
const long idwDialog::ID_STATICTEXT9 = wxNewId();
const long idwDialog::ID_TEXTCTRL3 = wxNewId();
const long idwDialog::ID_TEXTCTRL4 = wxNewId();
const long idwDialog::ID_TEXTCTRL5 = wxNewId();
const long idwDialog::ID_STATICTEXT10 = wxNewId();
const long idwDialog::ID_STATICTEXT11 = wxNewId();
const long idwDialog::ID_STATICTEXT12 = wxNewId();
const long idwDialog::ID_TEXTCTRL10 = wxNewId();
const long idwDialog::ID_TEXTCTRL6 = wxNewId();
const long idwDialog::ID_TEXTCTRL9 = wxNewId();
const long idwDialog::ID_STATICTEXT15 = wxNewId();
const long idwDialog::ID_TEXTCTRL7 = wxNewId();
const long idwDialog::ID_BITMAPBUTTON2 = wxNewId();
const long idwDialog::ID_STATICTEXT16 = wxNewId();
const long idwDialog::ID_BUTTON1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(idwDialog,wxDialog)
    //(*EventTable(idwDialog)
    //*)
END_EVENT_TABLE()

idwDialog::idwDialog(wxWindow* parent,wxWindowID id)
{
    //(*Initialize(idwDialog)
    wxFlexGridSizer* wxGSMain1;
    wxBoxSizer* BoxSizer1;

    Create(parent, wxID_ANY, _("wxWidgets app"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("wxID_ANY"));
    SetClientSize(wxSize(605,206));
    BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
    wxGSMain1 = new wxFlexGridSizer(10, 3, 0, 0);
    StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("Input file \n(x-y-z Text)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
    wxGSMain1->Add(StaticText1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtInputFileName = new wxTextCtrl(this, ID_TEXTCTRL1, _("*.dat, *.txt"), wxDefaultPosition, wxSize(342,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
    txtInputFileName->SetToolTip(_("Insert path and file name or choose via file selector"));
    wxGSMain1->Add(txtInputFileName, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    cmdOpenInput = new wxBitmapButton(this, ID_BITMAPBUTTON1, wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_BUTTON), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_BITMAPBUTTON1"));
    cmdOpenInput->SetDefault();
    wxGSMain1->Add(cmdOpenInput, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter1 = new wxStaticText(this, ID_STATICTEXT3, _("Power for IDW \ncalculation (1-8):"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT3"));
    wxGSMain1->Add(lblShifter1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblMaxYCoord = new wxStaticText(this, ID_STATICTEXT2, _("Max. Y Coord. input points:\nxxxx\nMax. Y Coord. Raster:"), wxDLG_UNIT(this,wxPoint(100,0)), wxDefaultSize, 0, _T("ID_STATICTEXT2"));
    wxGSMain1->Add(lblMaxYCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter2 = new wxStaticText(this, ID_STATICTEXT4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT4"));
    wxGSMain1->Add(lblShifter2, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtPowerIDW = new wxTextCtrl(this, ID_TEXTCTRL8, _("4"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL8"));
    wxGSMain1->Add(txtPowerIDW, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtMaxYCoord = new wxTextCtrl(this, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxSize(134,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
    wxGSMain1->Add(txtMaxYCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter32 = new wxStaticText(this, ID_STATICTEXT6, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT6"));
    wxGSMain1->Add(lblShifter32, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblMinXCoord = new wxStaticText(this, ID_STATICTEXT7, _("Min. X Coord. input points:\nxxxx\nMin. X Coord. Raster:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT7"));
    wxGSMain1->Add(lblMinXCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblRasterWidth = new wxStaticText(this, ID_STATICTEXT8, _("Raster width:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT8"));
    wxGSMain1->Add(lblRasterWidth, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblMaxXCoord = new wxStaticText(this, ID_STATICTEXT9, _("Max. X Coord. input points\nxxxx\nMax. X Coord. Raster:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT9"));
    wxGSMain1->Add(lblMaxXCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtMinXCoord = new wxTextCtrl(this, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxSize(122,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL3"));
    wxGSMain1->Add(txtMinXCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtRasterWidth = new wxTextCtrl(this, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL4"));
    wxGSMain1->Add(txtRasterWidth, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtMaxXCoord = new wxTextCtrl(this, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxSize(123,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL5"));
    wxGSMain1->Add(txtMaxXCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter61 = new wxStaticText(this, ID_STATICTEXT10, _("Search radius:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT10"));
    wxGSMain1->Add(lblShifter61, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblMinYCoord = new wxStaticText(this, ID_STATICTEXT11, _("Min. Y Coord. input points:\nxxxx\nMin. Y Coord. Raster:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT11"));
    wxGSMain1->Add(lblMinYCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter62 = new wxStaticText(this, ID_STATICTEXT12, _("Number of\n neighbors: \n(if all, set <0)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT12"));
    wxGSMain1->Add(lblShifter62, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtSearchRadius = new wxTextCtrl(this, ID_TEXTCTRL10, _("Text"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL10"));
    wxGSMain1->Add(txtSearchRadius, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtMinYCoord = new wxTextCtrl(this, ID_TEXTCTRL6, wxEmptyString, wxDefaultPosition, wxSize(117,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL6"));
    wxGSMain1->Add(txtMinYCoord, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtNumberNeighbors = new wxTextCtrl(this, ID_TEXTCTRL9, _("15"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL9"));
    wxGSMain1->Add(txtNumberNeighbors, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblResFile = new wxStaticText(this, ID_STATICTEXT15, _("Result file:\n(Surfer ASCII Grid)"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT15"));
    wxGSMain1->Add(lblResFile, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    txtResFile = new wxTextCtrl(this, ID_TEXTCTRL7, _("*.grd"), wxDefaultPosition, wxSize(339,21), 0, wxDefaultValidator, _T("ID_TEXTCTRL7"));
    wxGSMain1->Add(txtResFile, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    cmdSaveResult = new wxBitmapButton(this, ID_BITMAPBUTTON2, wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_BUTTON), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_BITMAPBUTTON2"));
    cmdSaveResult->SetDefault();
    wxGSMain1->Add(cmdSaveResult, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    lblShifter91 = new wxStaticText(this, ID_STATICTEXT16, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT16"));
    wxGSMain1->Add(lblShifter91, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    cmdRun = new wxButton(this, ID_BUTTON1, _("Run"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
    wxGSMain1->Add(cmdRun, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    BoxSizer1->Add(wxGSMain1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    SetSizer(BoxSizer1);
    fdOpenPointFile = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, _("*.txt, *.dat"), wxFD_DEFAULT_STYLE|wxFD_OPEN, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));
    fdSaveResult = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, _("*.grd"), wxFD_DEFAULT_STYLE|wxFD_SAVE, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));
    BoxSizer1->SetSizeHints(this);

    Connect(ID_BITMAPBUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&idwDialog::OnBitmapButton1Click);
    Connect(ID_BITMAPBUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&idwDialog::OncmdSaveResultClick);
    Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&idwDialog::OncmdRunClick);
    //*)
}

idwDialog::~idwDialog()
{
    //(*Destroy(idwDialog)
    //*)
}

void idwDialog::OnQuit(wxCommandEvent& event)
{
    Close();
}

void idwDialog::OnAbout(wxCommandEvent& event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Welcome to..."));
}

void idwDialog::OnBitmapButton1Click(wxCommandEvent& event)
{

    if (fdOpenPointFile->ShowModal()==wxID_OK)
    {
        long numInputPoints;
        double minX, maxX, minY, maxY, searchRadiusDefault;
        wxString wxStrMinX, wxStrMinY, wxStrMaxX, wxStrMaxY, wxStrSearchRad, tmpOpenFile;

        tmpOpenFile=fdOpenPointFile->GetPath();
        pathOpenPointFile=tmpOpenFile.mb_str();
        //wxMessageBox(pathOpenPointFile);
        txtInputFileName->Clear();
        txtInputFileName->AppendText(tmpOpenFile);

        getPoints readInput;
        readInput.setFile(pathOpenPointFile);
        readInput.getPointsFromFile();
        numInputPoints=readInput.getNumberInputPoints();


        readInput.analyzeFileContentXYZ();

        //wxString tmpString;
        //tmpString<<wxT("numPoints2= ");
        //wxMessageBox (tmpString<<numInputPoints);

        minX=readInput.getMinX();
        maxX=readInput.getMaxX();
        minY=readInput.getMinY();
        maxY=readInput.getMaxY();

        searchRadiusDefault=sqrt(((maxX-minX)*(maxX-minX))+((maxY-minY)*(maxY-minY)));
        searchRadiusDefault=searchRadiusDefault/4;


        lblMaxYCoord->SetLabel(wxStrMaxY<<maxY);
        lblMinYCoord->SetLabel(wxStrMinY<<minY);
        lblMaxXCoord->SetLabel(wxStrMaxX<<maxX);
        lblMinXCoord->SetLabel(wxStrMinX<<minX);
        txtSearchRadius->SetValue(wxStrSearchRad<<searchRadiusDefault);

        readInput.getInputData(xinput, yinput, zinput);
    }
}


void idwDialog::OncmdSaveResultClick(wxCommandEvent& event)
{


    if (fdSaveResult->ShowModal()==wxID_OK)
    {
        wxString wxstrTmpPath;
        wxstrTmpPath=fdSaveResult->GetPath();
        pathSaveResultFile=wxstrTmpPath.mb_str();
        txtResFile->Clear();
        txtResFile->AppendText(wxstrTmpPath);
    }
}

void idwDialog::OncmdRunClick(wxCommandEvent& event)
{
    wxString tmpString;
    double dblMaxX, dblMinX, dblMaxY, dblMinY, dblRasWidth, dblSearchRadius;
    long lngPowIDW, lngNumNeighborPoints;

    if(!txtMaxXCoord->GetValue().ToDouble(&dblMaxX)){/*error*/};
    if(!txtMinXCoord->GetValue().ToDouble(&dblMinX)){/*error*/};
    if(!txtMaxYCoord->GetValue().ToDouble(&dblMaxY)){/*error*/};
    if(!txtMinYCoord->GetValue().ToDouble(&dblMinY)){/*error*/};
    if(!txtRasterWidth->GetValue().ToDouble(&dblRasWidth)){/*error*/};
    if(!txtPowerIDW->GetValue().ToLong(&lngPowIDW)){/*error*/};
    if(!txtNumberNeighbors->GetValue().ToLong(&lngNumNeighborPoints)){/*error*/};
    if(!txtSearchRadius->GetValue().ToDouble(&dblSearchRadius)){/*error*/};

    calculate_IDW calcIDW;
    calcIDW.setRaster(dblMinX, dblMinY, dblMaxX, dblMaxY, dblRasWidth);
    calcIDW.setPointData(xinput,yinput,zinput);
//    calcIDW.calculateRadiusMeasPoints(lngPowIDW, dblSearchRadius);
    if (lngNumNeighborPoints<0){calcIDW.calculateAllMeasPoints(lngPowIDW);}
    else {calcIDW.calculateNumberMeasPoints(lngPowIDW, lngNumNeighborPoints, dblSearchRadius);}
    calcIDW.getRasterData(xras,yras,zras);
    numRasCols=calcIDW.getCols();
    numRasRows=calcIDW.getRows();


    //pathSaveResultFile="e:\\interpolation\\idw\\res.asc";
    exportRaster writeGrid;
    writeGrid.setRasterData(xras,yras,zras);
    writeGrid.writeGridFile(pathSaveResultFile, numRasCols, numRasRows, dblMinX, dblMinY, dblRasWidth);
    Close();
}

